﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("RealEncoding", "Describes a real vector encoding.")]
  [StorableClass]
  public sealed class RealEncoding : Encoding {

    [Storable]
    private IFixedValueParameter<IntValue> lengthParameter;
    public IFixedValueParameter<IntValue> LengthParameter {
      get { return lengthParameter; }
      set {
        if(value == null) throw new ArgumentNullException("Length parameter must not be null.");
        if (lengthParameter == value) return;
        lengthParameter = value;
        OnLengthParameterChanged();
      }
    }

    [Storable]
    private IValueParameter<DoubleMatrix> boundsParameter;
    public IValueParameter<DoubleMatrix> BoundsParameter {
      get { return boundsParameter; }
      set {
        if (value == null) throw new ArgumentNullException("Bounds parameter must not be null.");
        if (boundsParameter == value) return;
        boundsParameter = value;
        OnBoundsParameterChanged();
      }
    }

    public override IEnumerable<IValueParameter> EncodingParameters {
      get { return base.EncodingParameters.Concat(new IValueParameter[] { LengthParameter, BoundsParameter }); }
    }

    public int Length {
      get { return LengthParameter.Value.Value; }
      set { LengthParameter.Value.Value = value; }
    }
    public DoubleMatrix Bounds {
      get { return BoundsParameter.Value; }
      set { BoundsParameter.Value = value; }
    }

    public ISolutionCreator SolutionCreator {
      get {
        var creator = new UniformRandomRealVectorCreator();
        creator.RealVectorParameter.ActualName = Name;
        creator.LengthParameter.ActualName = LengthParameter.Name;
        creator.BoundsParameter.ActualName = BoundsParameter.Name;
        return creator;
      }
    }
      
    [StorableConstructor]
    private RealEncoding(bool deserializing) : base(deserializing) { }
    private RealEncoding(RealEncoding original, Cloner cloner)
      : base(original, cloner) {
      lengthParameter = cloner.Clone(original.lengthParameter);
      boundsParameter = cloner.Clone(original.boundsParameter);
    }
    public RealEncoding(string name, int length, double min, double max)
      : base(name) {
      if (min >= max) throw new ArgumentException("min must be less than max", "min");

      var bounds = new DoubleMatrix(1, 2);
      bounds[0, 0] = min;
      bounds[0, 1] = max;

      lengthParameter = new FixedValueParameter<IntValue>(Name + "Length", new IntValue(length));
      boundsParameter = new ValueParameter<DoubleMatrix>(Name + "Bounds", bounds);
    }
    public RealEncoding(string name, int length, IList<double> min, IList<double> max)
      : base(name) {
      if (min.Count == 0) throw new ArgumentException("Bounds must be given for the real parameters.");
      if (min.Count != max.Count) throw new ArgumentException("min must be of the same length as max", "min");
      if (min.Zip(max, (mi, ma) => mi >= ma).Any(x => x)) throw new ArgumentException("min must be less than max in each dimension", "min");

      var bounds = new DoubleMatrix(min.Count, 2);
      for (int i = 0; i < min.Count; i++) {
        bounds[i, 0] = min[i];
        bounds[i, 1] = max[i];
      }
      lengthParameter = new FixedValueParameter<IntValue>(Name + "Length", new IntValue(length));
      boundsParameter = new ValueParameter<DoubleMatrix>(Name + "Bounds", bounds);
    }  

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RealEncoding(this, cloner);
    }

    private void OnLengthParameterChanged() {
       //TODO rewire parameters
    }

    private void OnBoundsParameterChanged() {
      //TODO rewire parameters      
    }


  }
}                            